@@ -1,12 +1,12 @@
 // Wrapper for underlying C-language localization -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,19 +14,14 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 
-// You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// Under Section 7 of GPL version 3, you are granted additional
+// permissions described in the GCC Runtime Library Exception, version
+// 3.1, as published by the Free Software Foundation.
 
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// You should have received a copy of the GNU General Public License and
+// a copy of the GCC Runtime Library Exception along with this program;
+// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
+// <http://www.gnu.org/licenses/>.
 
 //
 // ISO C++ 14882: 22.8  Standard locale categories.
@@ -36,6 +31,7 @@
 
 #include <locale>
 #include <stdexcept>
+#include <limits>
 #include <langinfo.h>
 #include <bits/c++locale_internal.h>
 
@@ -47,12 +43,25 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 		   const __c_locale& __cloc)
     {
       char* __sanity;
-      float __f = __strtof_l(__s, &__sanity, __cloc);
-      if (__sanity != __s && __f != __builtin_huge_valf()
-	  && __f != -__builtin_huge_valf())
-	__v = __f;
-      else
-	__err |= ios_base::failbit;
+      __v = __strtof_l(__s, &__sanity, __cloc);
+
+      // _GLIBCXX_RESOLVE_LIB_DEFECTS
+      // 23. Num_get overflow result.
+      if (__sanity == __s || *__sanity != '\0')
+	{
+	  __v = 0.0f;
+	  __err = ios_base::failbit;
+	}
+      else if (__v == numeric_limits<float>::infinity())
+	{
+	  __v = numeric_limits<float>::max();
+	  __err = ios_base::failbit;
+	}
+      else if (__v == -numeric_limits<float>::infinity())
+	{
+	  __v = -numeric_limits<float>::max();
+	  __err = ios_base::failbit;
+	}
     }
 
   template<>
@@ -61,12 +70,25 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 		   const __c_locale& __cloc)
     {
       char* __sanity;
-      double __d = __strtod_l(__s, &__sanity, __cloc);
-      if (__sanity != __s && __d != __builtin_huge_val()
-	  && __d != -__builtin_huge_val())
-	__v = __d;
-      else
-	__err |= ios_base::failbit;
+      __v = __strtod_l(__s, &__sanity, __cloc);
+
+      // _GLIBCXX_RESOLVE_LIB_DEFECTS
+      // 23. Num_get overflow result.
+      if (__sanity == __s || *__sanity != '\0')
+	{
+	  __v = 0.0;
+	  __err = ios_base::failbit;
+	}
+      else if (__v == numeric_limits<double>::infinity())
+	{
+	  __v = numeric_limits<double>::max();
+	  __err = ios_base::failbit;
+	}
+      else if (__v == -numeric_limits<double>::infinity())
+	{
+	  __v = -numeric_limits<double>::max();
+	  __err = ios_base::failbit;
+	}
     }
 
   template<>
@@ -78,15 +100,28 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 #if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2)
       // Prefer strtold_l, as __strtold_l isn't prototyped in more recent
       // glibc versions.
-      long double __ld = strtold_l(__s, &__sanity, __cloc);
+      __v = strtold_l(__s, &__sanity, __cloc);
 #else
-      long double __ld = __strtold_l(__s, &__sanity, __cloc);
+      __v = __strtold_l(__s, &__sanity, __cloc);
 #endif
-      if (__sanity != __s && __ld != __builtin_huge_vall()
-	  && __ld != -__builtin_huge_vall())
-	__v = __ld;
-      else
-	__err |= ios_base::failbit;
+
+      // _GLIBCXX_RESOLVE_LIB_DEFECTS
+      // 23. Num_get overflow result.
+      if (__sanity == __s || *__sanity != '\0')
+	{
+	  __v = 0.0l;
+	  __err = ios_base::failbit;
+	}
+      else if (__v == numeric_limits<long double>::infinity())
+	{
+	  __v = numeric_limits<long double>::max();
+	  __err = ios_base::failbit;
+	}
+      else if (__v == -numeric_limits<long double>::infinity())
+	{
+	  __v = -numeric_limits<long double>::max();
+	  __err = ios_base::failbit;
+	}
     }
 
   void
